#include <stdio.h>
#include "mqtt_client.h"
#include "esp_log.h"
#include "nvs_flash.h"
#include "esp_wifi.h"
#include "esp_event.h"
#include "freertos/semphr.h"

// 需要把这两个修改成你家WIFI，测试是否连接成功
#define TEST_SSID "lgm"
#define TEST_PASSWORD "lgm5201718"

#define TAG "mqtt"

#define MQTT_ADDRESS "mqtt://192.168.110.129" // MQTT连接地址
#define MQTT_PORT 1883                        // MQTT连接端口号
#define MQTT_CLIENT "mqttx_d20250213"         // Client ID（设备唯一，大家最好自行改一下）
#define MQTT_USERNAME "hello1"                // MQTT用户名
#define MQTT_PASSWORD "12345678"              // MQTT密码

#define MQTT_PUBLIC_TOPIC "/test/topic1"    // 测试用的,推送消息主题
#define MQTT_SUBSCRIBE_TOPIC "/test/topic2" // 测试用的,需要订阅的主题

// MQTT客户端操作句柄
static esp_mqtt_client_handle_t mqtt_handle = NULL;

// 处理wifi的信号量
static SemaphoreHandle_t s_wifi_connect_sem = NULL;

/**
 * mqtt连接事件处理函数
 * @param event 事件参数
 * @return 无
 */
static void mqtt_event_handler(void *event_handler_arg,
                               esp_event_base_t event_base,
                               int32_t event_id,
                               void *event_data)
{
    esp_mqtt_event_handle_t data = (esp_mqtt_event_handle_t)event_data;
    // esp_mqtt_client_handle_t client = data->client;
    //  your_context_t *context = event->context;
    // switch ((esp_mqtt_event_id_t)event_id)
    switch (event_id)
    {
    case MQTT_EVENT_CONNECTED: // 连接成功
        ESP_LOGI(TAG, "mqtt connected");
        // s_is_mqtt_connected = true;
        // 连接成功后，订阅测试主题
        esp_mqtt_client_subscribe_single(mqtt_handle, MQTT_SUBSCRIBE_TOPIC, 1);
        break;
    case MQTT_EVENT_DISCONNECTED: // 连接断开
        ESP_LOGI(TAG, "mqtt disconnected");
        // s_is_mqtt_connected = false;
        break;
    case MQTT_EVENT_SUBSCRIBED: // 收到订阅消息ACK
        // ESP_LOGI(TAG, " mqtt subscribed ack, msg_id=%d", event->msg_id);
        break;
    case MQTT_EVENT_UNSUBSCRIBED: // 收到解订阅消息ACK
        ESP_LOGI(TAG, "收到解订阅消息ACK");
        break;
    case MQTT_EVENT_PUBLISHED: // 收到发布消息ACK
        ESP_LOGI(TAG, "收到发布消息ACK");
        // ESP_LOGI(TAG, "mqtt publish ack, msg_id=%d", event->msg_id);
        break;
    case MQTT_EVENT_DATA:
        ESP_LOGI(TAG, "topic->%s", data->topic);
        ESP_LOGI(TAG, "payload->%s", data->data);
        break;
    case MQTT_EVENT_ERROR:
        ESP_LOGI(TAG, "MQTT_EVENT_ERROR");
        break;
    default:
        break;
    }
}

/** 启动mqtt连接
 * @param 无
 * @return 无
 */
void mqtt_start(void)
{
    esp_mqtt_client_config_t mqtt_cfg = {0};
    mqtt_cfg.broker.address.uri = MQTT_ADDRESS;
    mqtt_cfg.broker.address.port = MQTT_PORT;
    // Client ID
    mqtt_cfg.credentials.client_id = MQTT_CLIENT;
    // 用户名
    mqtt_cfg.credentials.username = MQTT_USERNAME;
    // 密码
    mqtt_cfg.credentials.authentication.password = MQTT_PASSWORD;
    mqtt_cfg.session.disable_keepalive=false;
    mqtt_cfg.session.keepalive = 60;
    mqtt_cfg.session.disable_clean_session = false;
    mqtt_cfg.network.timeout_ms=10;


    ESP_LOGI(TAG, "mqtt connect->clientId:%s,username:%s,password:%s", mqtt_cfg.credentials.client_id,
             mqtt_cfg.credentials.username, mqtt_cfg.credentials.authentication.password);

    // 设置mqtt配置，返回mqtt操作句柄
    mqtt_handle = esp_mqtt_client_init(&mqtt_cfg);
    if (mqtt_handle != NULL)
    {

        ESP_LOGI(TAG, "MQTT客户端操作句柄");
    }

    // 注册mqtt事件回调函数
    esp_mqtt_client_register_event(mqtt_handle, ESP_EVENT_ANY_ID, mqtt_event_handler, NULL);

    // 启动mqtt连接
    esp_mqtt_client_start(mqtt_handle);
}

/** wifi事件回调函数
 * @param arg   用户传递的参数
 * @param event_base    事件类别
 * @param event_id      事件ID
 * @param event_data    事件携带的数据
 * @return 无
 */
static void event_handler(void *arg, esp_event_base_t event_base, int32_t event_id, void *event_data)
{
    if (event_base == WIFI_EVENT)
    {
        switch (event_id)
        {
        case WIFI_EVENT_STA_START: // WIFI以STA模式启动后触发此事件
            esp_wifi_connect();    // 启动WIFI连接
            break;
        case WIFI_EVENT_STA_CONNECTED: // WIFI连上路由器后，触发此事件
            ESP_LOGI(TAG, "connected to AP");
            break;
        case WIFI_EVENT_STA_DISCONNECTED: // WIFI从路由器断开连接后触发此事件
            esp_wifi_connect();           // 继续重连
            ESP_LOGI(TAG, "connect to the AP fail,retry now");
            break;
        default:
            break;
        }
    }
    else if (event_base == IP_EVENT) // IP相关事件
    {
        switch (event_id)
        {
        case IP_EVENT_STA_GOT_IP: // 只有获取到路由器分配的IP，才认为是连上了路由器
            // if(wifi_cb)
            //     wifi_cb(WIFI_CONNECTED);
            xSemaphoreGive(s_wifi_connect_sem);
            ESP_LOGI(TAG, "get ip address 路由器连接成功");
            break;
        }
    }
}

void app_main(void)
{
    ESP_ERROR_CHECK(nvs_flash_init());
    ESP_ERROR_CHECK(esp_netif_init());                // 用于初始化tcpip协议栈
    ESP_ERROR_CHECK(esp_event_loop_create_default()); // 创建一个默认系统事件调度循环，之后可以注册回调函数来处理系统的一些事件
    esp_netif_create_default_wifi_sta();              // 使用默认配置创建STA对象
    // 初始化WIFI
    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));

    // 初始化信号量
    s_wifi_connect_sem = xSemaphoreCreateBinary();

    // 注册事件
    ESP_ERROR_CHECK(esp_event_handler_register(WIFI_EVENT, ESP_EVENT_ANY_ID, &event_handler, NULL));
    ESP_ERROR_CHECK(esp_event_handler_register(IP_EVENT, IP_EVENT_STA_GOT_IP, &event_handler, NULL));

    // WIFI配置
    wifi_config_t wifi_config =
        {
            .sta =
                {
                    .ssid = TEST_SSID,                        // WIFI的SSID
                    .password = TEST_PASSWORD,                // WIFI密码
                    .threshold.authmode = WIFI_AUTH_WPA2_PSK, // 加密方式

                    .pmf_cfg =
                        {
                            .capable = true,
                            .required = false},
                },
        };

    // 启动WIFI
    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));               // 设置工作模式为STA
    ESP_ERROR_CHECK(esp_wifi_set_config(WIFI_IF_STA, &wifi_config)); // 设置wifi配置
    ESP_ERROR_CHECK(esp_wifi_start());                               // 启动WIFI
    xSemaphoreTake(s_wifi_connect_sem, portMAX_DELAY);
    mqtt_start();
}
